require 'test_helper'

class OcacheTest < ActiveSupport::TestCase
  setup do
    Rails.cache.write('init', true)
    Rails.cache.clear
    @options = {direction: true, created_at: DateTime.parse('january 7th 1986 00:00:00 +0000'), updated_at: DateTime.parse('january 7th 1986 00:00:00 +0000')}
  end

  test 'caching is enabled' do
    assert Rails.application.config.action_controller.perform_caching
  end

  test 'cache_fetch handles standard lib Class Methods' do
    assert_equal DateTime.cache_fetch(force: true).now.to_i, DateTime.now.to_i
  end

  # Module
  test 'cache_fetch handles options' do
    assert_equal Vote.find_by(direction: true), Vote.cache_fetch(force: true).find_by(direction: true)
    Vote.create(@options)
    assert_not_equal Vote.find_by(direction: true), Vote.cache_fetch.find_by(direction: true)
    assert_equal Vote.find_by(direction: true), Vote.cache_fetch(force: true).find_by(direction: true)

    assert_equal Rails.cache.read('votes/find_by/direction/true'), Vote.cache_fetch.find_by(direction: true)
  end

  test 'cache_fetch handles options and custom keys' do
    assert_equal Vote.find_by(direction: true), Vote.cache_fetch('custom_key', force: true).find_by(direction: true)
    Vote.create(@options)
    assert_not_equal Vote.find_by(direction: true), Vote.cache_fetch('custom_key').find_by(direction: true)
    assert_equal Vote.find_by(direction: true), Vote.cache_fetch('custom_key', force: true).find_by(direction: true)

    assert_equal Rails.cache.read('custom_key'), Vote.cache_fetch('custom_key').find_by(direction: true)
  end

  # Class Methods
  test 'cache_fetch handles class methods' do
    assert_equal 0, Vote.cache_fetch(force: true).expensive_class_method
    Vote.create(@options)
    assert_equal 0, Vote.cache_fetch.expensive_class_method
    assert_equal 1, Vote.cache_fetch(force: true).expensive_class_method

    assert_equal Rails.cache.read('votes/expensive_class_method'), Vote.cache_fetch.expensive_class_method
  end

  test 'cache_fetch handles class methods with custom keys' do
    assert_equal 0, Vote.cache_fetch('custom_key', force: true).expensive_class_method
    Vote.create(@options)
    assert_equal 0, Vote.cache_fetch('custom_key').expensive_class_method
    assert_equal 1, Vote.cache_fetch('custom_key', force: true).expensive_class_method

    assert_equal 1, Rails.cache.read('custom_key')
  end

  test 'cache_fetch handles options on class methods' do
    assert_equal 0, Vote.cache_fetch(force: true).expensive_class_method_with_options(direction: true)
    Vote.create(@options)
    assert_equal 0, Vote.cache_fetch.expensive_class_method_with_options(direction: true)
    assert_equal 1, Vote.cache_fetch(force: true).expensive_class_method_with_options(direction: true)

    assert_equal Rails.cache.read('votes/expensive_class_method_with_options/direction/true'), Vote.cache_fetch.expensive_class_method_with_options(direction: true)
  end

  test 'cache_fetch handles options on class methods with custom keys' do
    assert_equal 0, Vote.cache_fetch('custom_key', force: true).expensive_class_method_with_options(direction: true)
    Vote.create(@options)
    assert_equal 0, Vote.cache_fetch('custom_key').expensive_class_method_with_options(direction: true)
    assert_equal 1, Vote.cache_fetch('custom_key', force: true).expensive_class_method_with_options(direction: true)

    assert_equal 1, Rails.cache.read('custom_key')
  end

  test 'cache_fetch handles arguments on class methods' do
    assert_equal 0, Vote.cache_fetch(force: true).expensive_class_method_with_arguments(true)
    Vote.create(@options)
    assert_equal 0, Vote.cache_fetch.expensive_class_method_with_arguments(true)
    assert_equal 1, Vote.cache_fetch(force: true).expensive_class_method_with_arguments(true)

    assert_equal Rails.cache.read('votes/expensive_class_method_with_arguments/true'), Vote.cache_fetch.expensive_class_method_with_arguments(true)
  end

  test 'cache_fetch handles arguments on class methods with custom keys' do
    assert_equal 0, Vote.cache_fetch('custom_key', force: true).expensive_class_method_with_arguments(true)
    Vote.create(@options)
    assert_equal 0, Vote.cache_fetch('custom_key').expensive_class_method_with_arguments(true)
    assert_equal 1, Vote.cache_fetch('custom_key', force: true).expensive_class_method_with_arguments(true)

    assert_equal 1, Rails.cache.read('custom_key')
  end

  # Instance Methods
  test 'cache_fetch handles instance methods' do
    Vote.create(@options)
    assert_equal 1, Vote.take.cache_fetch(force: true).expensive_instance_method
    Vote.create(@options)
    assert_equal 1, Vote.take.cache_fetch.expensive_instance_method
    assert_equal 2, Vote.take.cache_fetch(force: true).expensive_instance_method

    assert_equal Rails.cache.read('votes/1-19860107000000000000000/expensive_instance_method'), Vote.take.cache_fetch.expensive_instance_method
  end

  test 'cache_fetch handles instance methods with custom keys' do
    Vote.create(@options)
    assert_equal 1, Vote.take.cache_fetch('custom_key', force: true).expensive_instance_method
    Vote.create(@options)
    assert_equal 1, Vote.take.cache_fetch('custom_key').expensive_instance_method
    assert_equal 2, Vote.take.cache_fetch('custom_key', force: true).expensive_instance_method

    assert_equal 2, Rails.cache.read('custom_key')
  end

  test 'cache_fetch handles options on instance methods' do
    Vote.create(@options)
    assert_equal 1, Vote.take.cache_fetch(force: true).expensive_instance_method_with_options(direction: true)
    Vote.create(@options)
    assert_equal 1, Vote.take.cache_fetch.expensive_instance_method_with_options(direction: true)
    assert_equal 2, Vote.take.cache_fetch(force: true).expensive_instance_method_with_options(direction: true)

    assert_equal Rails.cache.read('votes/1-19860107000000000000000/expensive_instance_method_with_options/direction/true'), Vote.take.cache_fetch.expensive_instance_method_with_options(direction: true)
  end

  test 'cache_fetch handles options on instance methods with custom keys' do
    Vote.create(@options)
    assert_equal 1, Vote.take.cache_fetch('custom_key', force: true).expensive_instance_method_with_options(direction: true)
    Vote.create(@options)
    assert_equal 1, Vote.take.cache_fetch('custom_key').expensive_instance_method_with_options(direction: true)
    assert_equal 2, Vote.take.cache_fetch('custom_key', force: true).expensive_instance_method_with_options(direction: true)

    assert_equal 2, Rails.cache.read('custom_key')
  end

  test 'cache_fetch handles arguments on instance methods' do
    Vote.create(@options)
    assert_equal 1, Vote.take.cache_fetch(force: true).expensive_instance_method_with_arguments(true)
    Vote.create(@options)
    assert_equal 1, Vote.take.cache_fetch.expensive_instance_method_with_arguments(true)
    assert_equal 2, Vote.take.cache_fetch(force: true).expensive_instance_method_with_arguments(true)

    assert_equal Rails.cache.read('votes/1-19860107000000000000000/expensive_instance_method_with_arguments/true'), Vote.take.cache_fetch.expensive_instance_method_with_arguments(true)

  end

  test 'cache_fetch handles arguments on instance methods with custom keys' do
    Vote.create(@options)
    assert_equal 1, Vote.take.cache_fetch('custom_key', force: true).expensive_instance_method_with_arguments(true)
    Vote.create(@options)
    assert_equal 1, Vote.take.cache_fetch('custom_key').expensive_instance_method_with_arguments(true)
    assert_equal 2, Vote.take.cache_fetch('custom_key', force: true).expensive_instance_method_with_arguments(true)

    assert_equal 2, Rails.cache.read('custom_key')
  end
end
